import { MinusOutlined, PlusOutlined } from '@ant-design/icons';
import { useBoolean, useEventEmitter, useUpdate } from 'ahooks';
import React, {
  createContext,
  forwardRef,
  useEffect,
  useImperativeHandle,
  useLayoutEffect,
  useRef,
} from 'react';
import { calculateBoundingCircle } from '../utils';
import styles from './index.less';
const MAP_BASE_URL = 'https://api.cdmbc.cn:4432/gateway/gis/1/';

export const MapViewerContext = createContext<any>(null);
/** 3方图层服务 */
const layersOptionsList = [
  // {
  //   name: '行政区划-街道',
  //   code: '8e33910897f9471daed85b4460f3eb82',
  // },
  {
    name: '蓝色底图-成都市',
    layer: 'CimBaseMap_Vector_LSB_202312',
    code: 'cfeb25a614e64f5ba258edaeca44f268',
    TileMatrixSet: 'ChinaPublicServicesCGCS2000_CimBaseMap_Vector_LSB_202312',
  },
  // {
  //   name: '行政区划-社区',
  //   layer: 'CDXZQH_CSJ',
  //   code: 'a4ee6112c4774fa4a21f367ef977687e',
  //   TileMatrixSet: 'ChinaPublicServicesCGCS2000_CDXZQH_CSJ',
  // },

  // {
  //   name: '行政区划-区县级',
  //   layer: 'CDXZQH_QXJ',
  //   code: 'e2fc91076d9c46feb691b6687c43f72e',
  //   TileMatrixSet: 'ChinaPublicServicesCGCS2000_CDXZQH_QXJ',
  // },
  // {
  //   name: '蓝色底图-东部新区',
  //   layer: 'layer',
  //   code: '998cb64aa9a5491fa3a77a73ae120e5e',
  //   TileMatrixSet: 'ChinaPublicServicesCGCS2000_layer',
  // },
];

/**  @name 地图实例 */
const MapView: React.ForwardRefRenderFunction<
  MapViewMethods,
  React.PropsWithChildren<MapViewProps>
> = (props, ref) => {
  const {
    zoom,
    center,
    children,
    showComponent = false,
    showCustomComponent = true,
    boundingSpherePoints,
    mode = '3D',
  } = props;

  const mapRef = useRef<HTMLDivElement>(null);
  /** 地图实例 */
  const mapViewer = useRef<any>(null);
  const update = useUpdate();
  const click$ = useEventEmitter();

  const [isReady, { setFalse, setTrue }] = useBoolean(false);

  useEffect(() => {
    if (isReady) update();
    /** 销毁地图 */
    return () => {
      // mapViewer.current?.destroy?.();
      // mapViewer.current = null;
    };
  }, [isReady]);
  /**
   * @description 暴露给父组件的地图实例
   */
  useImperativeHandle(ref, () => mapViewer.current);

  /** 地图初始化 */
  const initMap = () => {
    if (!center) {
      throw new Error('地图中心点是必须的');
    }
    /** 设置高程 */
    let z = center.z ?? 200000;
    /**根据缩放等级，更新 z 值 */
    if (zoom && zoom < 20 && zoom > 0) {
      z = 20000 * zoom;
    }

    mapViewer.current = new CooGL.MapViewer(mapRef.current, {
      showComponent: showComponent,
      defaultView: {
        destination: {
          x: center.x,
          y: center.y,
          z: z,
        },
        orientation: {
          heading: 6.283054179776913,
          pitch: -Math.PI / 2,
          roll: 0,
        },
      },
    });

    layersOptionsList.forEach((item) => {
      /** 添加图层 */
      mapViewer.current.addLayer(
        new CooGL.Layer.fromWmtsImagery({
          url: `${MAP_BASE_URL}${item.code}?AppKey=657676357265784832&SERVICE=WMTS&REQUEST=GetTile&VERSION=1.0.0&Layer=${item.layer}&Style=default&Format=image/png&TileMatrixSet=${item.TileMatrixSet}&TileMatrix={TileMatrix}&TileCol={TileCol}&TileRow={TileRow}`,
          layer: item.layer,
          style: 'default',
          format: 'image/png',
          tileMatrixSetID: item.TileMatrixSet,
          tilingScheme: new CooGL.GeographicTilingScheme2({}),
          maximumLevel: 18,
        }),
      );
    });
    /** 设置地图模式 */
    mode === '2D'
      ? mapViewer.current?.scene.morphTo2D(0)
      : mapViewer.current?.scene.morphTo3D(0);

    /** 地图加载完成后，在加载覆盖物 */
    mapViewer.current.readyEvent.addEventListener(() => {
      setTrue();
    });
    mapViewer.current.event.pick(function (mapEventTarget: any) {
      if (mapEventTarget) {
        /** 说明 */
        if (mapEventTarget.layer instanceof CooGL.ImageGraphic) {
          click$.emit(mapEventTarget.layer);
        }
      }
    });
  };

  useLayoutEffect(() => {
    initMap();
  }, []);

  /** 地图层级放大 */
  const zoomIn = () => {
    mapViewer.current?.camera.zoom(0.5);
  };

  /** 设置地图视野，显示所有覆盖物 */
  const setViewport = () => {
    if (boundingSpherePoints) {
      /** 设置高程 */
      let z = center.z ?? 200000;
      /**根据缩放等级，更新 z 值 */
      if (zoom && zoom < 20 && zoom > 0) {
        z = 20000 * zoom;
      }
      const info = calculateBoundingCircle(boundingSpherePoints);
      mapViewer.current?.camera.flyToBoundingSphere(
        {
          x: info.center[0],
          y: info.center[1],
          z: z,
        },
        info.radius,
        {
          offset: {
            heading: 6.283054179776913,
            pitch: -Math.PI / 2,
            roll: 0,
          },
        },
      );
    }
  };

  /** 地图层级缩小 */
  const zoomOut = () => {
    mapViewer.current?.camera.zoom(2);
  };

  return (
    <MapViewerContext.Provider value={{ mapViewer: mapViewer.current, click$ }}>
      <div className={styles.container}>
        <div ref={mapRef} className={styles.map} id="customMap" />
        {isReady && children}
        {showCustomComponent && (
          <div className={styles.mapController}>
            {boundingSpherePoints && (
              <div onClick={() => setViewport()}>
                <img
                  src={require('@/assets/images/map/mapReload.png')}
                  alt=""
                />
              </div>
            )}
            <div onClick={zoomIn}>
              <PlusOutlined></PlusOutlined>
            </div>
            <div onClick={zoomOut}>
              <MinusOutlined />
            </div>
          </div>
        )}
      </div>
    </MapViewerContext.Provider>
  );
};

export default forwardRef(MapView);
